@@ -21,7 +21,6 @@
 
 // KDE
 #include <KLocalizedString>
-#include <KMessageBox>
 #include <KRandom>
 
 // KMahjongg
@@ -36,6 +35,9 @@
 #include "movelistanimation.h"
 #include "prefs.h"
 #include "selectionanimation.h"
+#include "gamebackground.h"
+#include "gameremovedtiles.h"
+
 
 GameView::GameView(GameScene * gameScene, GameData * gameData, QWidget * parent)
     : QGraphicsView(gameScene, parent)
@@ -44,8 +46,12 @@ GameView::GameView(GameScene * gameScene, GameData * gameData, QWidget * parent)
     , m_gamePaused(false)
     , m_match(false)
     , m_gameGenerated(false)
+    , m_showRemovedTiles(true)
+    , m_remTilesWidthFactor(0.3)
     , m_gameData(gameData)
     , m_selectedItem(nullptr)
+    , m_gameBackground(new GameBackground())
+    , m_gameRemovedTiles(new GameRemovedTiles())
     , m_tilesetPath(new QString())
     , m_backgroundPath(new QString())
     , m_helpAnimation(new SelectionAnimation(this))
@@ -56,12 +62,15 @@ GameView::GameView(GameScene * gameScene, GameData * gameData, QWidget * parent)
 {
     // Some settings to the QGraphicsView.
     setFocusPolicy(Qt::NoFocus);
-    setStyleSheet("QGraphicsView { border-style: none; }");
+    setStyleSheet(QStringLiteral("QGraphicsView { border-style: none; }"));
     setAutoFillBackground(true);
 
     // Read in some settings.
     m_angle = static_cast<TileViewAngle>(Prefs::angle());
 
+    // Show removed tiles.
+    m_showRemovedTiles = Prefs::removedTiles();
+
     // Init HelpAnimation
     m_helpAnimation->setAnimationSpeed(ANIMATION_SPEED);
     m_helpAnimation->setRepetitions(3);
@@ -72,6 +81,17 @@ GameView::GameView(GameScene * gameScene, GameData * gameData, QWidget * parent)
     // Init MoveListAnimation
     m_moveListAnimation->setAnimationSpeed(ANIMATION_SPEED);
 
+    // Set the tileset to the game removed tiles object.
+    m_gameRemovedTiles->setTileset(m_tiles);
+    m_gameRemovedTiles->setGameData(m_gameData);
+    m_gameRemovedTiles->setVisible(m_showRemovedTiles);
+
+    // Add the fix background item to the scene
+    scene()->setBackgroundItem(m_gameBackground);
+
+    // Add the fix removedtiles item to the scene
+    scene()->setRemovedTilesItem(m_gameRemovedTiles);
+
     m_selectionChangedConnect = connect(scene(), &GameScene::selectionChanged, this, &GameView::selectionChanged);
 
     connect(m_demoAnimation, &DemoAnimation::changeItemSelectedState, this, &GameView::changeItemSelectedState);
@@ -88,6 +108,7 @@ GameView::~GameView()
     delete m_helpAnimation;
     delete m_demoAnimation;
     delete m_moveListAnimation;
+    delete m_gameBackground;
     delete m_background;
     delete m_backgroundPath;
     delete m_tilesetPath;
@@ -136,6 +157,9 @@ bool GameView::undo()
 
         ++m_gameData->m_allowRedo;
 
+        // Undo removed tile object.
+        m_gameRemovedTiles->undo();
+
         setStatusText(i18n("Undo operation done successfully."));
 
         return true;
@@ -189,6 +213,9 @@ void GameView::createNewGame(long gameNumber)
     checkDemoAnimationActive(true);
     checkMoveListAnimationActive(true);
 
+    // Reset the removed tile object.
+    m_gameRemovedTiles->reset();
+
     // Create a random game number, if no one was given.
     if (gameNumber == -1) {
         m_gameNumber = KRandom::random();
@@ -219,6 +246,9 @@ void GameView::createNewGame(long gameNumber)
             addItemsFromBoardLayout();
             populateItemNumber();
 
+            QResizeEvent event(size(), size());
+            resizeEvent(&event);
+
             setStatusText(i18n("Ready. Now it is your turn."));
 
             return;
@@ -229,7 +259,7 @@ void GameView::createNewGame(long gameNumber)
     m_gameGenerated = false;
 
     // Hide all generated tiles.
-    foreach (GameItem * item, items()) {
+    foreach (GameItem * item, getGameItems()) {
         item->hide();
     }
 
@@ -309,6 +339,10 @@ void GameView::removeItem(POSITION & stItemPos)
     // Put an empty item in the data object. (data part)
     m_gameData->putTile(stItemPos.z, stItemPos.y, stItemPos.x, 0);
 
+    // Add the tile to the removedtiles object.
+    m_gameRemovedTiles->addItem(stItemPos);
+    m_gameRemovedTiles->update();
+
     // Remove the item from the scene object. (graphic part)
     scene()->removeItem(stItemPos);
 
@@ -465,11 +499,11 @@ bool GameView::validMovesAvailable(bool silent)
 void GameView::pause(bool isPaused)
 {
     if (isPaused) {
-        foreach (GameItem * item, items()) {
+        foreach (GameItem * item, getGameItems()) {
             item->hide();
         }
     } else {
-        foreach (GameItem * item, items()) {
+        foreach (GameItem * item, getGameItems()) {
             item->show();
         }
     }
@@ -496,7 +530,7 @@ void GameView::shuffle()
     m_gameData->shuffle();
 
     // Update the item images.
-    updateItemsImages(items());
+    updateItemsImages(getGameItems());
 
     // Cause of using the shuffle function... increase the cheat used variable.
     m_cheatsUsed += 15;
@@ -527,8 +561,8 @@ void GameView::addItemsFromBoardLayout()
     // The signal is reconnected at the end of the function.
     disconnect(m_selectionChangedConnect);
 
-    // Remove all existing items.
-    scene()->clear();
+    // Remove all GameItem objects
+    scene()->clearGameItems();
 
     // Create the items and add them to the scene.
     for (int iZ = 0; iZ < m_gameData->m_depth; ++iZ) {
@@ -550,7 +584,7 @@ void GameView::addItemsFromBoardLayout()
         }
     }
 
-    updateItemsImages(items());
+    updateItemsImages(getGameItems());
     updateItemsOrder();
 
     // Reconnect our selectionChanged() slot.
@@ -600,26 +634,61 @@ void GameView::addItemAndUpdate(POSITION & stItemPos)
     addItem(stItemPos, true, true, true);
 }
 
+void GameView::showRemovedTiles(bool show)
+{
+    if (m_showRemovedTiles != show) {
+        m_showRemovedTiles = show;
+        m_gameRemovedTiles->setVisible(show);
+
+        QResizeEvent event(size(), size());
+        resizeEvent(&event);
+        updateItemsPosition();
+    }
+}
+
+void GameView::updateItemsPosition()
+{
+    updateItemsPosition(scene()->items());
+}
+
 void GameView::updateItemsPosition(const QList<GameItem *> &gameItems)
 {
+    // The width and height need to be corrected, related to the removedtiles
+    // view and wether it is shown or not. For now the removed tiles field can
+    // only be placed to the right of the board.
+    qreal boardWidth = (!m_showRemovedTiles) ? width() : width() * (1 - m_remTilesWidthFactor - 0.05);
+    qreal boardHeight = height();
+
+    // TODO: Change!!!
+    // Make decision of painting the removed tiles.
+
     // These factor are needed for the different angles. So we simply can
     // calculate to move the items to the left or right and up or down.
     int angleXFactor = (m_angle == NE || m_angle == SE) ? -1 : 1;
     int angleYFactor = (m_angle == NW || m_angle == NE) ? -1 : 1;
 
     // Get half width and height of tile faces: minimum spacing = 1 pixel.
-    qreal tileWidth = m_tiles->qWidth() + 0.5;
-    qreal tileHeight = m_tiles->qHeight() + 0.5;
+    // NOTE - qWidth is devided by 2 in kmahjonggtileset.cpp. The reason is 
+    //        unknown for now. Please review this later.
+    qreal tileFaceWidth = m_tiles->qWidth() * 2 + 1;
+    qreal tileFaceHeight = m_tiles->qHeight() * 2 + 1;
 
     // Get half height and width of tile-layout: ((n - 1) faces + full tile)/2.
-    qreal tilesWidth = tileWidth * (m_gameData->m_width - 2) / 2
-        + m_tiles->width() / 2;
-    qreal tilesHeight = tileHeight * (m_gameData->m_height - 2) / 2
-        + m_tiles->height() / 2;
+    
+    // Because the positions of the tiles can be half-positioned, the width and
+    // height in GameData is two times higher. To get the maximum number of
+    // tiles in a row or column, the GameData-width and -height have to be
+    // devided by two.
+    qreal numTilesX = m_gameData->m_width / 2;
+    qreal numTilesY = m_gameData->m_height / 2;
+
+    // Calculate the total width and height of board layout with tiles.
+    qreal tilesWidth = tileFaceWidth * numTilesX + m_tiles->levelOffsetX();
+    qreal tilesHeight = tileFaceHeight * numTilesY + m_tiles->levelOffsetY();
 
     // Get the top-left offset required to center the items in the view.
-    qreal xFrame = (width() / 2 - tilesWidth) / 2;
-    qreal yFrame = (height() / 2 - tilesHeight) / 2;
+    qreal xFrame = (boardWidth - tilesWidth) / 2;
+    qreal yFrame = (boardHeight - tilesHeight) / 2;
 
     // TODO - The last /2 makes it HALF what it should be, but it gets doubled
     //        somehow before the view is painted. Why? Apparently it is because
@@ -627,8 +696,11 @@ void GameView::updateItemsPosition(const QList<GameItem *> &gameItems)
     //        being an item in the scene and filling the scene completely. So
     //        the whole scene is just the rectangle that contains the tiles.
     // NOTE - scene()->itemsBoundingRect() returns the correct doubled offset.
-
-    for (int i = 0; i < gameItems.size(); ++i) {
+    // NOTE - insert game background object but problem persist
+    // SOLVED - Problem was in coordinate system in QGraphicsItem/GameItem.
+    //          Painting positions are relative, but were defined absolute to
+    //          scene coordinate system. Same with boundingRect().
+    for (int i = 0; i < gameItems.size(); i++) {
         GameItem * gameItem = gameItems.at(i);
 
         // Get rasterized positions of the item.
@@ -639,11 +711,17 @@ void GameView::updateItemsPosition(const QList<GameItem *> &gameItems)
 
         // Set the position of the item on the scene.
         gameItem->setPos(
-            xFrame + tileWidth * x / 2
-                + z * angleXFactor * (m_tiles->levelOffsetX() / 2),
-            yFrame + tileHeight * y / 2
-                + z * angleYFactor * (m_tiles->levelOffsetY() / 2));
+            xFrame + tileFaceWidth / 2 * x
+                + z * angleXFactor * (m_tiles->levelOffsetX()),
+            yFrame + tileFaceHeight / 2 * y
+                + z * angleYFactor * (m_tiles->levelOffsetY()));
     }
+
+    // Position the removedtiles object.
+    qreal removedTilesBorder = height() * 0.1 / 2;
+    m_gameRemovedTiles->setPos(
+        width() * (1 - m_remTilesWidthFactor), removedTilesBorder
+    );
 }
 
 void GameView::updateItemsOrder()
@@ -703,7 +781,7 @@ void GameView::updateItemsOrder()
         }
     }
 
-    updateItemsPosition(items());
+    updateItemsPosition(getGameItems());
 }
 
 void GameView::orderLine(GameItem * startItem, int xStart, int xEnd, int xCounter, int y, int yCounter, int z, int & zCount)
@@ -786,7 +864,7 @@ bool GameView::setBackgroundPath(QString const & backgroundPath)
 void GameView::setAngle(TileViewAngle angle)
 {
     m_angle = angle;
-    updateItemsImages(items());
+    updateItemsImages(getGameItems());
     updateItemsOrder();
 }
 
@@ -812,7 +890,7 @@ void GameView::angleSwitchCCW()
             break;
     }
 
-    updateItemsImages(items());
+    updateItemsImages(getGameItems());
     updateItemsOrder();
 }
 
@@ -833,17 +911,20 @@ void GameView::angleSwitchCW()
             break;
     }
 
-    updateItemsImages(items());
+    updateItemsImages(getGameItems());
     updateItemsOrder();
 }
 
-QList<GameItem *> GameView::items() const
+QList<GameItem *> GameView::getGameItems() const
 {
-    QList<QGraphicsItem *> originalList = QGraphicsView::items();
+    QList<QGraphicsItem *> items = QGraphicsView::items();
     QList<GameItem *> tmpList;
 
-    for (int i = 0; i < originalList.size(); ++i) {
-        tmpList.append(dynamic_cast<GameItem *>(originalList.at(i)));
+    for (int i = 0; i < items.size(); ++i) {
+        GameItem *gameItem = dynamic_cast<GameItem *>(items.at(i));
+        if (nullptr != gameItem) {
+            tmpList.append(gameItem);
+        }
     }
 
     return tmpList;
@@ -876,8 +957,28 @@ void GameView::resizeEvent(QResizeEvent * event)
         return;
     }
 
-    resizeTileset(event->size());
+    // The size need to be corrected, related to the removedtiles
+    // view and wether it is shown or not. For now the removed tiles field can
+    // only be placed to the right of the board.
+    QSize size(event->size());
+    if (m_showRemovedTiles) {
+        resizeTileset(QSize(
+            size.width() * (1 - m_remTilesWidthFactor - 0.05), size.height()
+        ));
 
+        // Update removed tiles
+        qreal removedTilesBorder = size.height() * 0.1 / 2;
+        m_gameRemovedTiles->setSize(
+            size.width() * m_remTilesWidthFactor - removedTilesBorder, 
+            size.height() * 0.9
+        );
+    } else {
+        resizeTileset(QSize(
+            size.width(), size.height()
+        ));
+    }
+
+    // Update background
     m_background->sizeChanged(width(), height());
     updateBackground();
 
@@ -890,16 +991,18 @@ void GameView::resizeTileset(const QSize & size)
         return;
     }
 
-    QSize newtiles = m_tiles->preferredTileSize(size, m_gameData->m_width / 2, m_gameData->m_height / 2);
+    QSize newtiles = m_tiles->preferredTileSize(
+        size, m_gameData->m_width / 2, m_gameData->m_height / 2
+    );
 
-    foreach (GameItem * item, items()) {
+    foreach (GameItem * item, getGameItems()) {
         item->prepareForGeometryChange();
     }
 
     m_tiles->reloadTileset(newtiles);
 
-    updateItemsImages(items());
-    updateItemsPosition(items());
+    updateItemsImages(getGameItems());
+    updateItemsPosition(getGameItems());
 }
 
 void GameView::updateItemsImages(const QList<GameItem *> &gameItems)
@@ -938,11 +1041,8 @@ void GameView::setStatusText(QString const & text)
 
 void GameView::updateBackground()
 {
-    // qCDebug(KMAHJONGG_LOG) << "Update the background";
-    // TODO - The background should be a scene-item? See updateItemsPosition().
-
-    QBrush brush(m_background->getBackground());
-    setBackgroundBrush(brush);
+    m_gameBackground->setBackground(m_background->getBackground());
+    m_gameBackground->setSize(width(), height());
 }
 
 void GameView::setGameData(GameData * gameData)
