@@ -27,6 +27,8 @@ static const int ANIMATION_SPEED = 200;
 class GameScene;
 class GameData;
 class GameItem;
+class GameBackground;
+class GameRemovedTiles;
 class SelectionAnimation;
 class MoveListAnimation;
 class DemoAnimation;
@@ -59,6 +61,11 @@ public:
      * @param gameItems The items of which the positions should be updated. */
     void updateItemsPosition(const QList<GameItem *> &gameItems);
 
+    /**
+     * Overloaded function of scene game item positioning.
+     */
+    void updateItemsPosition();
+
     /**
      * Updates the whole widget.
      *
@@ -67,7 +74,13 @@ public:
 
     /**
      * Override from QGraphcisView. */
-    virtual QList<GameItem *> items() const;
+    virtual QList<GameItem *> getGameItems() const;
+
+    /**
+     * Set wether removed tiles should be shown.
+     * @param show True if removed tiles should be shown.
+     */
+    void showRemovedTiles(bool show);
 
     /**
      * Override from QGraphicsView. */
@@ -397,9 +410,14 @@ private:
     bool m_gamePaused;
     bool m_match;
     bool m_gameGenerated;
+    bool m_showRemovedTiles;
+
+    qreal m_remTilesWidthFactor;
 
     GameData * m_gameData;
     GameItem * m_selectedItem;
+    GameBackground * m_gameBackground;
+    GameRemovedTiles * m_gameRemovedTiles;
 
     QString * m_tilesetPath;
     QString * m_backgroundPath;
